/*
** Copyright (C) 2007 Jesse Chappell <jesse@essej.net>
**  
** This program is free software; you can redistribute it and/or modify
** it under the terms of the GNU General Public License as published by
** the Free Software Foundation; either version 2 of the License, or
** (at your option) any later version.
**  
** This program is distributed in the hope that it will be useful,
** but WITHOUT ANY WARRANTY; without even the implied warranty of
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
** GNU General Public License for more details.
**  
** You should have received a copy of the GNU General Public License
** along with this program; if not, write to the Free Software
** Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
**  
*/

#ifndef __ac3jack_alsa_spdifwriter__
#define __ac3jack_alsa_spdifwriter__

#include <string>
#include "boost/thread/thread.hpp"
#include "boost/thread/condition.hpp"
#include <cstdio>

#include <alsa/asoundlib.h>

#include "types.hpp"
#include "spdif_writer.hpp"
#include "ringbuffer.hpp"

namespace AC3JACK
{

class AlsaSpdifWriter
	: public SpdifWriter
{
  public:

	AlsaSpdifWriter(unsigned int samplerate=48000,
			const std::string & cardstr = "",
		        size_t bufsize = 32768);
	virtual ~AlsaSpdifWriter();
	
	virtual int get_devices(DeviceInfoList & retlist);

	virtual void set_device (std::string devstr);
	virtual std::string get_device () { return _cardstr; }

	virtual void set_allow_pcm (bool val);
	virtual bool get_allow_pcm () const { return _allow_pcm; }

	/**
	 *   queues data to be written to the spdif.
	 *   this method is realtime safe
	 */
	virtual size_t write(unsigned char * data, size_t length);

	virtual void start();
	
  protected:

	virtual void run_work_thread();

	void write_to_spdif(size_t minblocksize);

	void wait_for_more(boost::try_mutex::scoped_lock & lk);

	bool open_device();
	void close_device();
	
	int output_play (short * output_samples, nframes_t num_frames);

	void probe_devices();
	
	std::string                    _cardstr;
	RingBuffer<unsigned char>    * _buffer;
	unsigned int                   _samplerate;

	snd_pcm_t * _pcm;

	bool                           _allow_pcm;
	DeviceInfoList                 _devlist;
	
};



	

}


#endif
