/*
** Copyright (C) 2006 Jesse Chappell <jesse@essej.net>
**  
** This program is free software; you can redistribute it and/or modify
** it under the terms of the GNU General Public License as published by
** the Free Software Foundation; either version 2 of the License, or
** (at your option) any later version.
**  
** This program is distributed in the hope that it will be useful,
** but WITHOUT ANY WARRANTY; without even the implied warranty of
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
** GNU General Public License for more details.
**  
** You should have received a copy of the GNU General Public License
** along with this program; if not, write to the Free Software
** Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
**  
*/

#include <iostream>
#include <cstring>

#include "encoder.hpp"

using namespace AC3JACK;
using namespace std;

Encoder::Encoder()
{
	_initialized = false;

	aften_set_defaults (&_aften_context);
}


Encoder::~Encoder()
{
	if (_initialized) {
		aften_encode_close (&_aften_context);
	}
}

bool Encoder::set_params(const EncoderParams & params)
{
	// convert to aften params
	_aften_context.channels = params.channels;
	_aften_context.samplerate = params.sample_rate;
	_aften_context.params.bitrate = params.bitrate;

	int acmod = A52_ACMOD_MONO;
	int lfe = params.lfe;

	switch(params.channels) {
	case 1: acmod = A52_ACMOD_MONO; break;
	case 2: acmod = A52_ACMOD_STEREO; break;
	case 3: acmod = A52_ACMOD_3_0; break;
	case 4: acmod = A52_ACMOD_2_2; break;
	case 5: acmod = A52_ACMOD_3_2; break;
		break;
	default: break;
	}

	if (lfe) {
		_aften_context.channels += 1;
	}
	
	_aften_context.acmod = acmod;
	_aften_context.lfe = lfe;
	_aften_context.sample_format = A52_SAMPLE_FMT_FLT;
	_aften_context.verbose = 1;

	_aften_context.system.n_threads = 1;

	// TODO; deal with crashes on some platforms when simd is on
#ifndef __APPLE__
	memset(&_aften_context.system.wanted_simd_instructions, 0, sizeof(AftenSimdInstructions));
#endif

	return true;
}

bool Encoder::initialize()
{

	if (aften_encode_init (&_aften_context) != 0) {
		cerr << "Error initializing aften encoder" << endl;
		_initialized = false;
	}
	else {
		_initialized = true;

	}
	return _initialized;
}

int Encoder::run(unsigned char * ac3buf, const sample_t * samplebuf, nframes_t count)
{
	if (!_initialized) return -1;

	// return number bytes written to ac3buf, or -1 on error
	int ret;

	ret = aften_encode_frame (&_aften_context, ac3buf, samplebuf, count);

	return ret;
}

