/*
** Copyright (C) 2007 Jesse Chappell <jesse@essej.net>
**  
** This program is free software; you can redistribute it and/or modify
** it under the terms of the GNU General Public License as published by
** the Free Software Foundation; either version 2 of the License, or
** (at your option) any later version.
**  
** This program is distributed in the hope that it will be useful,
** but WITHOUT ANY WARRANTY; without even the implied warranty of
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
** GNU General Public License for more details.
**  
** You should have received a copy of the GNU General Public License
** along with this program; if not, write to the Free Software
** Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
**  
*/


#ifndef __ac3jack_filewriter__
#define __ac3jack_filewriter__

#include <string>
#include "boost/thread/thread.hpp"
#include "boost/thread/condition.hpp"
#include <cstdio>

//#include "types.hpp"

#include "thread_writer.hpp"
#include "ringbuffer.hpp"

namespace AC3JACK
{

class FileWriter
	: public ThreadWriter
{
  public:

	FileWriter(const std::string & filename, size_t bufsize);
	virtual ~FileWriter();

	/**
	 *   queues data to be written to the file.
	 *   this method is realtime safe
	 */
	virtual size_t write(unsigned char * data, size_t length);

	
  protected:

	virtual void run_work_thread();

	void write_to_disk(size_t minblocksize);
	void wait_for_more(boost::try_mutex::scoped_lock & lk);

	bool open_output_file();
	
	std::string                    _filename;
	RingBuffer<unsigned char>    * _buffer;

	std::FILE *               _outfile;
	
};



	

}


#endif
