/*
** Copyright (C) 2007 Jesse Chappell <jesse@essej.net>
**  
** This program is free software; you can redistribute it and/or modify
** it under the terms of the GNU General Public License as published by
** the Free Software Foundation; either version 2 of the License, or
** (at your option) any later version.
**  
** This program is distributed in the hope that it will be useful,
** but WITHOUT ANY WARRANTY; without even the implied warranty of
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
** GNU General Public License for more details.
**  
** You should have received a copy of the GNU General Public License
** along with this program; if not, write to the Free Software
** Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
**  
*/

#include "connections_popup.hpp"
#include "osc_client.hpp"

using namespace AC3JACKGui;
using namespace std;

BEGIN_EVENT_TABLE(ConnectionsPopup, wxMenu)
	EVT_MENU_RANGE (0, 100, ConnectionsPopup::OnSelectItem)
END_EVENT_TABLE()

enum {
	ID_DisconnectAll = 0
};
	

ConnectionsPopup::ConnectionsPopup (wxWindow *parent, OscClient * osc, const wxString &chan)
	:  wxMenu(), _osc(osc), _chan(chan)
{
	wxMenuItem * item = 0;
	int itemid = 1;
	int last;
	wxString connstr;
	wxString availstr;	

	// todo get available conns
	
	// get all connections for channel
	_osc->get_value(wxT("input_connectable_ports"), availstr);
	if (!_osc->get_value(wxString::Format(wxT("input_connections/%s"), chan.c_str()), connstr)) {
		cerr << "no conn string for: " << wxString::Format(wxT("input_connections/%s"), chan.c_str()).ToAscii() << endl;
	}

	// cerr << "conn string for " << chan.ToAscii() << ": " << connstr.ToAscii() << endl;
	int pos = 0;
	int endpos;
	
	if ((endpos = availstr.find(wxT("\n"))) == -1) {
		endpos = availstr.size();
	}
	
			      
	while (endpos > pos) {
		wxString line = availstr.substr(pos, endpos-pos);

		item = AppendCheckItem (itemid++, line);

		if (item && connstr.find(line) != -1) {
			item->Check(true);
		}
		
		if (endpos >= availstr.size()) {
			break;
		}
		pos = endpos + 1;
		
		if ((endpos = availstr.find(wxT("\n"), pos)) == -1) {
			endpos = availstr.size();
		}
	}

	AppendSeparator();
	item = new wxMenuItem(this, ID_DisconnectAll, wxT("<Disconnect All>"));
	Append (item);
}

ConnectionsPopup::~ConnectionsPopup()
{
	// cerr << "popup dying" << endl;
}

void ConnectionsPopup::OnSelectItem(wxCommandEvent &event)
{
	int id = event.GetId();

	wxString label = GetLabel(id);
	// cerr << "selected: " << id << "  label: " << label.ToAscii() << endl;
	int count = GetMenuItemCount();
	wxString connstr;

	if (id == ID_DisconnectAll) {
		// intentionally left blank
		connstr = wxT("");
	}
	else {
		for (int i=0 ; i < count; ++i) {
			wxMenuItem * item = FindItemByPosition(i);
			if (item && item->IsChecked()) {
				connstr.Append(item->GetLabel());
				connstr.Append(wxT("\n"));
			}
		}
	}

	// cerr << "conn str is: " << connstr.ToAscii() << endl;
	_osc->post_param_change(wxString::Format(wxT("input_connections/%s"), _chan.c_str()), connstr);
}
