/*
** Copyright (C) 2004 Jesse Chappell <jesse@essej.net>
**  
** This program is free software; you can redistribute it and/or modify
** it under the terms of the GNU General Public License as published by
** the Free Software Foundation; either version 2 of the License, or
** (at your option) any later version.
**  
** This program is distributed in the hope that it will be useful,
** but WITHOUT ANY WARRANTY; without even the implied warranty of
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
** GNU General Public License for more details.
**  
** You should have received a copy of the GNU General Public License
** along with this program; if not, write to the Free Software
** Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
**  
*/

#include <wx/wx.h>
#include <wx/file.h>
#include <wx/filename.h>
#include <wx/image.h>
#include <wx/utils.h>
#include <wx/dir.h>
#include <wx/spinctrl.h>
#include <wx/splash.h>

#include <iostream>
#include <cstdio>
#include <cmath>

#include "version.h"

#include "main_panel.hpp"
#include "gui_app.hpp"
#include "osc_client.hpp"
#include "connections_popup.hpp"

using namespace AC3JACK;
using namespace AC3JACKGui;
using namespace std;

enum {
	ID_UpdateTimer = 9000,
	ID_AboutMenu,
	ID_HelpTipsMenu,
	ID_PreferencesMenu,
	ID_ConnectionMenu,
	ID_LoadSession,
	ID_SaveSession,
	ID_Quit,
	ID_JackOutCheck,
	ID_ChannelChoice,
	ID_BitrateChoice,
	ID_SpdifOutCheck,
	ID_SpdifPcmCheck,
	ID_SpdifDevChoice,
	ID_FileOutCheck,
	ID_FilenameEntry,
	ID_BrowseButton,
	ID_DurationLimitCheck,
	ID_DurationEntry,
	ID_FilesizeText,
	ID_DurationText,
	ID_JackTransportCheck,
	ID_LfeCheck,
	ID_SpdifStatusText,
	ID_LeftConnButton,
	ID_RightConnButton,
	ID_CenterConnButton,
	ID_LeftSurrConnButton,
	ID_RightSurrConnButton,
	ID_LfeConnButton
       
};


BEGIN_EVENT_TABLE(MainPanel, wxPanel)

	EVT_IDLE(MainPanel::OnIdle)
	EVT_SIZE(MainPanel::OnSize)
	EVT_PAINT(MainPanel::OnPaint)
	EVT_TIMER(ID_UpdateTimer, MainPanel::OnUpdateTimer)

	EVT_ACTIVATE (MainPanel::OnActivate)
	EVT_ACTIVATE_APP (MainPanel::OnActivate)

	EVT_CHECKBOX (ID_JackOutCheck, MainPanel::on_check)
	EVT_CHECKBOX (ID_LfeCheck, MainPanel::on_check)
	EVT_CHECKBOX (ID_FileOutCheck, MainPanel::on_check)
	EVT_CHECKBOX (ID_DurationLimitCheck, MainPanel::on_check)
	EVT_CHECKBOX (ID_JackTransportCheck, MainPanel::on_check)
	EVT_CHECKBOX (ID_SpdifOutCheck, MainPanel::on_check)
	EVT_CHECKBOX (ID_SpdifPcmCheck, MainPanel::on_check)

	EVT_CHOICE (ID_ChannelChoice, MainPanel::on_choice)
	EVT_CHOICE (ID_SpdifDevChoice, MainPanel::on_choice)
	EVT_COMBOBOX (ID_BitrateChoice, MainPanel::on_choice)		

	EVT_TEXT_ENTER (ID_FilenameEntry, MainPanel::on_text)
	EVT_TEXT_ENTER (ID_DurationEntry, MainPanel::on_text)	

	EVT_BUTTON (ID_BrowseButton, MainPanel::on_button)
	EVT_BUTTON (ID_LeftConnButton, MainPanel::on_button)
	EVT_BUTTON (ID_CenterConnButton, MainPanel::on_button)
	EVT_BUTTON (ID_RightConnButton, MainPanel::on_button)
	EVT_BUTTON (ID_LeftSurrConnButton, MainPanel::on_button)
	EVT_BUTTON (ID_RightSurrConnButton, MainPanel::on_button)
	EVT_BUTTON (ID_LfeConnButton, MainPanel::on_button)
	
END_EVENT_TABLE()

	MainPanel::MainPanel(wxWindow * parent, wxWindowID id, const wxPoint& pos, const wxSize& size)
	: wxPanel(parent, id, pos, size)

{
	_got_new_data = 0;
	_engine_alive = true;
	
	_rcdir = wxGetHomeDir() + wxFileName::GetPathSeparator() + wxT(".ac3jack");

	// load_rc();
	
	init();

	// must start osc control after init
	_osc_control = new OscClient(_rcdir);


	_update_timer = new wxTimer(this, ID_UpdateTimer);
	_update_timer->Start(11000, true);

	_osc_control->ConnectFailed.connect (slot (*this,  &MainPanel::on_connect_failed));
	_osc_control->LostConnection.connect (slot (*this,  &MainPanel::on_connection_lost));
	_osc_control->IsAlive.connect (slot (*this,  &MainPanel::on_engine_alive));
	_osc_control->ErrorReceived.connect (slot (*this,  &MainPanel::on_error_received));
	_osc_update_connection = _osc_control->NewDataReady.connect (slot (*this, &MainPanel::osc_data_ready));



	
}

MainPanel::~MainPanel()
{
	save_rc();
	
	delete _update_timer;
	delete _osc_control;
}

void
MainPanel::init()
{
	//_main_sizer = new wxBoxSizer(wxVERTICAL);
	_topsizer = new wxBoxSizer(wxVERTICAL);

	//wxBoxSizer * rowsizer = new wxBoxSizer(wxHORIZONTAL);
	wxInitAllImageHandlers();
	
	//SetBackgroundColour(*wxBLACK);
	//SetThemeEnabled(false);
	
	wxFont smallFont = *wxSMALL_FONT;

	_top_panel = new wxPanel(this);
	//_top_panel->SetThemeEnabled(false);
	//_top_panel->SetBackgroundColour(*wxBLACK);

	wxBoxSizer * topcolsizer = new wxBoxSizer(wxVERTICAL);
	

	wxBoxSizer * upper_sizer = new wxBoxSizer(wxHORIZONTAL);
	
	// AC3 Options
	wxStaticBox * shotBox = new wxStaticBox(_top_panel, -1, wxT("AC3 Options"), wxDefaultPosition, wxDefaultSize);
        wxStaticBoxSizer * colsizer = new wxStaticBoxSizer(shotBox, wxVERTICAL);
	
	wxBoxSizer * rowsizer = new wxBoxSizer(wxHORIZONTAL);
	
	wxStaticText * statText = new wxStaticText(_top_panel, -1, wxT("Channels:"));
	rowsizer->Add (statText, 0, wxALIGN_RIGHT|wxALIGN_CENTRE_VERTICAL|wxALL, 2);
	
	_channel_choice = new wxChoice(_top_panel, ID_ChannelChoice);
	_channel_choice->Append(wxT("5 - Left/Center/Right/L.Surr/R.Surr"), (void *) (long) 5);
	_channel_choice->Append(wxT("4 - Left/Right/L.Surr/R.Surr"),        (void *) (long) 4);
	_channel_choice->Append(wxT("3 - Left/Center/Right"),               (void *) (long) 3);
	_channel_choice->Append(wxT("2 - Left/Right"),                      (void *) (long) 2);
	
	rowsizer->Add (_channel_choice, 0, wxALL|wxALIGN_CENTRE_VERTICAL, 2);

	_lfe_check = new wxCheckBox(_top_panel, ID_LfeCheck, wxT("LFE"));
	rowsizer->Add (_lfe_check, 0, wxALL|wxALIGN_CENTRE_VERTICAL, 2);
	
	colsizer->Add (rowsizer, 1, wxEXPAND|wxALL, 2); 


	rowsizer = new wxBoxSizer(wxHORIZONTAL);
	statText = new wxStaticText(_top_panel, -1, wxT("Bitrate:"));
	rowsizer->Add (statText, 0, wxALIGN_RIGHT|wxALIGN_CENTRE_VERTICAL|wxALL, 2);
	
	_bitrate_choice = new wxComboBox(_top_panel, ID_BitrateChoice);
	_bitrate_choice->Append(wxT("640"));
	_bitrate_choice->Append(wxT("448"));
	_bitrate_choice->Append(wxT("320"));
	_bitrate_choice->Append(wxT("256"));
	_bitrate_choice->Append(wxT("128"));
	_bitrate_choice->Append(wxT("32"));	

	rowsizer->Add (_bitrate_choice, 0, wxALL|wxALIGN_CENTRE_VERTICAL, 2);
	
	colsizer->Add (rowsizer, 1, wxEXPAND|wxALL, 2); 
	upper_sizer->Add(colsizer, 0, wxEXPAND|wxALL, 2);

	// connection buttons
	shotBox = new wxStaticBox(_top_panel, -1, wxT("Connections"), wxDefaultPosition, wxDefaultSize);
        wxStaticBoxSizer * boxsizer = new wxStaticBoxSizer(shotBox, wxVERTICAL);

	int buttwidth = 120;
	rowsizer = new wxBoxSizer(wxHORIZONTAL);
	
	wxBoxSizer * unitsizer = new wxBoxSizer(wxVERTICAL);
        statText = new wxStaticText(_top_panel, -1, wxT("Left"));
	statText->SetFont(smallFont);
	unitsizer->Add (statText, 0, wxALIGN_CENTRE|wxALIGN_CENTRE_VERTICAL|wxALL, 2);
	_left_conn_button = new wxButton(_top_panel, ID_LeftConnButton, wxT("<Not Connected>"), wxDefaultPosition, wxSize(buttwidth, -1));
	_left_conn_button->SetFont(smallFont);
	unitsizer->Add(_left_conn_button, 0, wxALL|wxALIGN_CENTRE|wxEXPAND, 1);
	rowsizer->Add(unitsizer, 1, wxALIGN_CENTRE);

	unitsizer = new wxBoxSizer(wxVERTICAL);
        statText = new wxStaticText(_top_panel, -1, wxT("Center"));
	statText->SetFont(smallFont);
	unitsizer->Add (statText, 0, wxALIGN_CENTRE|wxALIGN_CENTRE_VERTICAL|wxALL, 2);
	_center_conn_button = new wxButton(_top_panel, ID_CenterConnButton, wxT("<Not Connected>"), wxDefaultPosition, wxSize(buttwidth, -1));
	_center_conn_button->SetFont(smallFont);
	unitsizer->Add(_center_conn_button, 0, wxALL|wxALIGN_CENTRE|wxEXPAND, 1);
	rowsizer->Add(unitsizer, 1, wxALIGN_CENTRE);

	unitsizer = new wxBoxSizer(wxVERTICAL);
        statText = new wxStaticText(_top_panel, -1, wxT("Right"));
	statText->SetFont(smallFont);
	unitsizer->Add (statText, 0, wxALIGN_CENTRE|wxALIGN_CENTRE_VERTICAL|wxALL, 2);
	_right_conn_button = new wxButton(_top_panel, ID_RightConnButton, wxT("<Not Connected>"), wxDefaultPosition, wxSize(buttwidth, -1));
	_right_conn_button->SetFont(smallFont);
	unitsizer->Add(_right_conn_button, 0, wxALL|wxALIGN_CENTRE|wxEXPAND, 1);
	rowsizer->Add(unitsizer, 1, wxALIGN_CENTRE);
	
	boxsizer->Add(rowsizer, 0, wxEXPAND|wxALL, 1);

	rowsizer = new wxBoxSizer(wxHORIZONTAL);
	
	unitsizer = new wxBoxSizer(wxVERTICAL);
        statText = new wxStaticText(_top_panel, -1, wxT("L. Surr"));
	statText->SetFont(smallFont);
	unitsizer->Add (statText, 0, wxALIGN_CENTRE|wxALIGN_CENTRE_VERTICAL|wxALL, 2);
	_leftsurr_conn_button = new wxButton(_top_panel, ID_LeftSurrConnButton, wxT("<Not Connected>"), wxDefaultPosition, wxSize(buttwidth, -1));
	_leftsurr_conn_button->SetFont(smallFont);
	unitsizer->Add(_leftsurr_conn_button, 0, wxALL|wxALIGN_CENTRE|wxEXPAND, 1);
	rowsizer->Add(unitsizer, 1, wxALIGN_CENTRE);

	unitsizer = new wxBoxSizer(wxVERTICAL);
        statText = new wxStaticText(_top_panel, -1, wxT("LFE"));
	statText->SetFont(smallFont);
	unitsizer->Add (statText, 0, wxALIGN_CENTRE|wxALIGN_CENTRE_VERTICAL|wxALL, 2);
	_lfe_conn_button = new wxButton(_top_panel, ID_LfeConnButton, wxT("<Not Connected>"), wxDefaultPosition, wxSize(buttwidth, -1));
	_lfe_conn_button->SetFont(smallFont);
	unitsizer->Add(_lfe_conn_button, 0, wxALL|wxALIGN_CENTRE|wxEXPAND, 1);
	rowsizer->Add(unitsizer, 1, wxALIGN_CENTRE);

	unitsizer = new wxBoxSizer(wxVERTICAL);
        statText = new wxStaticText(_top_panel, -1, wxT("R. Surr"));
	statText->SetFont(smallFont);
	unitsizer->Add (statText, 0, wxALIGN_CENTRE|wxALIGN_CENTRE_VERTICAL|wxALL, 2);
	_rightsurr_conn_button = new wxButton(_top_panel, ID_RightSurrConnButton, wxT("<Not Connected>"), wxDefaultPosition, wxSize(buttwidth, -1));
	_rightsurr_conn_button->SetFont(smallFont);
	unitsizer->Add(_rightsurr_conn_button, 0, wxALL|wxALIGN_CENTRE|wxEXPAND, 1);
	rowsizer->Add(unitsizer, 1, wxALIGN_CENTRE);
	
	boxsizer->Add(rowsizer, 0, wxEXPAND|wxALL, 1);
	
	
	upper_sizer->Add(boxsizer, 1, wxALL|wxEXPAND, 2);

	
	topcolsizer->Add (upper_sizer, 0, wxALL|wxEXPAND, 4);


	// Realtime output Options
	shotBox = new wxStaticBox(_top_panel, -1, wxT("Realtime Output Options"), wxDefaultPosition, wxDefaultSize);
        colsizer = new wxStaticBoxSizer(shotBox, wxVERTICAL);

	rowsizer = new wxBoxSizer(wxHORIZONTAL);
	_jack_out_check = new wxCheckBox(_top_panel, ID_JackOutCheck, wxT("Output AC3 as Raw SPDIF PCM on JACK Ports (see tooltip)"));
	_jack_out_check->SetToolTip(wxT(" This option is for audio interfaces that have SPDIF but don't advertise AC3 passthrough to the OS or that don't allow its use simultaneously with JACK.  You must connect the L and R ports to a pair of hardware channels that route to the SPDIF output. WARNING: Do NOT connect these jack ports to analog channels with speakers attached, or damage might occur! It is wise to keep your target receiver muted until it locks on the AC3 stream."));
	rowsizer->Add(_jack_out_check, 0, wxALL|wxALIGN_CENTRE_VERTICAL, 2);
	colsizer->Add (rowsizer, 1, wxEXPAND|wxALL, 2); 

	rowsizer = new wxBoxSizer(wxHORIZONTAL);
	_spdif_out_check = new wxCheckBox(_top_panel, ID_SpdifOutCheck, wxT("Output to SPDIF Device"));
	rowsizer->Add(_spdif_out_check, 0, wxALL|wxALIGN_CENTRE_VERTICAL, 2);
	rowsizer->Add(6,1,0);
	_spdif_device_choice = new wxChoice(_top_panel, ID_SpdifDevChoice);
	rowsizer->Add(_spdif_device_choice, 1, wxALL|wxALIGN_CENTRE_VERTICAL, 2);
	rowsizer->Add(6, 1, 0);
	//colsizer->Add (rowsizer, 1, wxEXPAND|wxALL, 2); 

	//rowsizer = new wxBoxSizer(wxHORIZONTAL);
	//rowsizer->Add(50, 1, 0);
	_spdif_pcm_check = new wxCheckBox(_top_panel, ID_SpdifPcmCheck, wxT("Disguise as PCM (WARNING! See tooltip)"));
	_spdif_pcm_check->SetToolTip(wxT(" WARNING: Do NOT choose an analog output with speakers attached, or damage might occur. It is wise to keep your target receiver muted until it locks on the AC3 stream.  This option is for audio interfaces that have SPDIF but don't advertise AC3 passthrough to the OS.  The device/channel list is populated with all available when this option is checked."));
	_spdif_pcm_check->SetFont(smallFont);
	rowsizer->Add(_spdif_pcm_check, 0, wxALL|wxALIGN_CENTRE_VERTICAL, 2);

	colsizer->Add (rowsizer, 1, wxEXPAND|wxALL, 2); 

	rowsizer = new wxBoxSizer(wxHORIZONTAL);
	rowsizer->Add(15, 1, 0);
	statText = new wxStaticText(_top_panel, -1, wxT("Status:"));
	rowsizer->Add (statText, 0, wxALIGN_RIGHT|wxALIGN_CENTRE_VERTICAL|wxALL, 2);
	_spdif_status_text = new wxTextCtrl(_top_panel, ID_SpdifStatusText, wxT(""), wxDefaultPosition, wxSize(80,-1), wxTE_READONLY);
	rowsizer->Add(_spdif_status_text, 1, wxALL|wxALIGN_CENTRE_VERTICAL, 2);	
	
	colsizer->Add (rowsizer, 1, wxEXPAND|wxALL, 2); 
	

	topcolsizer->Add (colsizer, 0, wxALL|wxEXPAND, 4);

	
	// File output Options
	shotBox = new wxStaticBox(_top_panel, -1, wxT("File Output Options"), wxDefaultPosition, wxDefaultSize);
        colsizer = new wxStaticBoxSizer(shotBox, wxVERTICAL);

	rowsizer = new wxBoxSizer(wxHORIZONTAL);
	_file_out_check = new wxCheckBox(_top_panel, ID_FileOutCheck, wxT("Output to File"));
	rowsizer->Add(_file_out_check, 0, wxALL|wxALIGN_CENTRE_VERTICAL, 2);
	_filename_entry = new wxTextCtrl(_top_panel, ID_FilenameEntry, wxT(""), wxDefaultPosition, wxDefaultSize, wxTE_PROCESS_ENTER);
	rowsizer->Add(_filename_entry, 1, wxALL|wxALIGN_CENTRE_VERTICAL, 2);	
	_browse_button = new wxButton(_top_panel, ID_BrowseButton, wxT("Browse..."));
	rowsizer->Add(_browse_button, 0, wxALL|wxALIGN_CENTRE_VERTICAL, 2);	
	colsizer->Add (rowsizer, 1, wxEXPAND|wxALL, 2); 

	rowsizer = new wxBoxSizer(wxHORIZONTAL);
	_duration_limit_check = new wxCheckBox(_top_panel, ID_DurationLimitCheck, wxT("Duration Limit (secs):"));
	rowsizer->Add(_duration_limit_check, 0, wxALL|wxALIGN_CENTRE_VERTICAL, 2);
	_duration_entry = new wxTextCtrl(_top_panel, ID_DurationEntry, wxT(""), wxDefaultPosition, wxSize(80,-1), wxTE_PROCESS_ENTER);
	rowsizer->Add(_duration_entry, 1, wxALL|wxALIGN_CENTRE_VERTICAL, 2);	

	statText = new wxStaticText(_top_panel, -1, wxT("Curr Size:"));
	rowsizer->Add (statText, 0, wxALIGN_RIGHT|wxALIGN_CENTRE_VERTICAL|wxALL, 2);
	_filesize_text = new wxTextCtrl(_top_panel, ID_FilesizeText, wxT(""), wxDefaultPosition, wxSize(80,-1), wxTE_READONLY);
	rowsizer->Add(_filesize_text, 0, wxALL|wxALIGN_CENTRE_VERTICAL, 2);	
	statText = new wxStaticText(_top_panel, -1, wxT("Elapsed Time:"));
	rowsizer->Add (statText, 0, wxALIGN_RIGHT|wxALIGN_CENTRE_VERTICAL|wxALL, 2);
	_duration_text = new wxTextCtrl(_top_panel, ID_DurationText, wxT(""), wxDefaultPosition, wxSize(80,-1), wxTE_READONLY);
	rowsizer->Add(_duration_text, 0, wxALL|wxALIGN_CENTRE_VERTICAL, 2);	
	colsizer->Add (rowsizer, 1, wxEXPAND|wxALL, 2); 

	rowsizer = new wxBoxSizer(wxHORIZONTAL);
	_jack_transport_check = new wxCheckBox(_top_panel, ID_JackTransportCheck, wxT("Output only when JACK transport is rolling"));
	rowsizer->Add(_jack_transport_check, 0, wxALL|wxALIGN_CENTRE_VERTICAL, 2);
	colsizer->Add (rowsizer, 1, wxEXPAND|wxALL, 2); 
	
	topcolsizer->Add (colsizer, 0, wxALL|wxEXPAND, 4);

	





	

	_top_panel->SetSizer( topcolsizer );      // actually set the sizer
	topcolsizer->Fit( _top_panel );            // set size to minimum size as calculated by the sizer
	topcolsizer->SetSizeHints( _top_panel );   // set size hints to honour mininum size

	_topsizer->Add (_top_panel, 1, wxEXPAND);

	
	//_scroller = new wxScrolledWindow(this, -1, wxDefaultPosition, wxDefaultSize, wxVSCROLL);
	//_scroller->SetBackgroundColour(*wxBLACK);
	


	//_topsizer->Add (_scroller, 1, wxEXPAND);
	
	//_scroller->SetSizer( _main_sizer );      // actually set the sizer
	//_scroller->SetAutoLayout( true );     // tell dialog to use sizer

	//_scroller->SetScrollRate (0, 30);
	//_scroller->EnableScrolling (true, true);

	//_main_sizer->Fit( _scroller );            // set size to minimum size as calculated by the sizer
	// _main_sizer->SetSizeHints( _scroller );   // set size hints to honour mininum size

	
	this->SetAutoLayout( true );     // tell dialog to use sizer
	this->SetSizer( _topsizer );      // actually set the sizer
	_topsizer->Fit( this );            // set size to minimum size as calculated by the sizer
	_topsizer->SetSizeHints( this );   // set size hints to honour mininum size

	// _scroller->SetFocus();
}


void
MainPanel::osc_data_ready()
{
	// cerr << "osc ready" << endl;
	// this is called from another thread
	_got_new_data++;

	::wxWakeUpIdle();
}

void 
MainPanel::on_connect_failed (const std::string & msg)
{
	//wxMessageDialog dial(this, wxString::FromAscii(msg.c_str()), wxT("Connection Error"), wxOK);
	//dial.SetTitle(wxT("Connection Error"));
	//dial.ShowModal();

	
}

void 
MainPanel::on_connection_lost (const std::string & msg)
{
	wxMessageDialog dial(this, wxString::FromAscii(msg.c_str()), wxT("Lost Connection"), wxOK);
	dial.SetTitle(wxT("Lost Connection"));

	dial.ShowModal();
}

void
MainPanel::on_error_received (const std::string & msg)
{
	wxMessageDialog dial(this, wxString::FromAscii(msg.c_str()), wxT("Error Received"), wxOK);
	dial.SetTitle(wxT("Error Received"));

	dial.ShowModal();
}


void
MainPanel::on_engine_alive ()
{
	// cerr << "got alive ping" << endl;
	_engine_alive = true;
}


void
MainPanel::OnUpdateTimer(wxTimerEvent &ev)
{
	// check to see if our connected server is still alive
	
	if (_osc_control->connected()) {
 		_osc_control->update_values();

		if (!_engine_alive) {		
			_osc_control->disconnect();
			on_connection_lost ("Lost connection to remote AC3Jack engine.");
		}

		_engine_alive = false;
		_osc_control->send_alive_ping();
	}

	_update_timer->Start(11000, true);
}

void
MainPanel::do_close(bool quitengine)
{
	// send quit command to looper by default
	_osc_update_connection.disconnect();

// sleep for a short period before stopping engine
#if wxCHECK_VERSION(2,5,3)
	::wxMilliSleep(500);
#else
	::wxUsleep(500);
#endif

	if (quitengine) {
		_osc_control->send_quit();
	}
}


void
MainPanel::OnActivate(wxActivateEvent &ev)
{

	ev.Skip();
}


void MainPanel::time_to_string(double time, wxString & retval)
{
	double hours, tmp, mins, secs;
	tmp = modf(time / 3600.0, &hours);
	tmp = modf(tmp * 60, &mins);
	secs = tmp * 60;
	retval = wxString::Format(wxT("%02d:%02d:%02d"), (int) hours, (int)mins, (int)secs);
}

void
MainPanel::update_controls()
{
	// get recent controls from loop control
	int ival;
	double dval;
	wxString sval;
	int64_t i64val;
	
	if (_osc_control->is_updated(wxT("curr_file_duration"))) {
		_osc_control->get_value(wxT("curr_file_duration"), dval);
		if (dval > 0.0) {
			wxString timestr;
			time_to_string(dval, timestr);		
			_duration_text->SetValue(timestr);
		} else {
			_duration_text->SetValue(wxT(""));
		}
	}

	if (_osc_control->is_updated(wxT("curr_file_size"))) {
		_osc_control->get_value(wxT("curr_file_size"), i64val);
		if (i64val == 0) {
			_filesize_text->SetValue(wxT(""));
		}
		else if (i64val > 1000000) {
			_filesize_text->SetValue(wxString::Format(wxT("%.2f MB"), i64val / 1000000.0));
		}
		else {
			_filesize_text->SetValue(wxString::Format(wxT("%.1f kB"), i64val / 1000.0));
		}
	}

	if (_osc_control->is_updated(wxT("jack_out"))) {
		_osc_control->get_value(wxT("jack_out"), ival);
		_jack_out_check->SetValue(ival ? true: false);
	}

	if (_osc_control->is_updated(wxT("jack_transport"))) {
		_osc_control->get_value(wxT("jack_transport"), ival);
		_jack_transport_check->SetValue(ival ? true: false);
	}

	if (_osc_control->is_updated(wxT("spdif_out"))) {
		_osc_control->get_value(wxT("spdif_out"), ival);
		_spdif_out_check->SetValue(ival ? true: false);
	}

	if (_osc_control->is_updated(wxT("spdif_pcm"))) {
		_osc_control->get_value(wxT("spdif_pcm"), ival);
		_spdif_pcm_check->SetValue(ival ? true: false);
	}

	if (_osc_control->is_updated(wxT("outfile_name"))) {
		_osc_control->get_value(wxT("outfile_name"), sval);
		_filename_entry->SetValue(sval);
	}

	if (_osc_control->is_updated(wxT("spdif_card"))) {
		_osc_control->get_value(wxT("spdif_card"), sval);
		set_spdif_dev_choice(sval);
	}

	if (_osc_control->is_updated(wxT("encoder/bitrate"))) {
		_osc_control->get_value(wxT("encoder/bitrate"), ival);
		_bitrate_choice->SetValue(wxString::Format(wxT("%d"), ival));
	}

	if (_osc_control->is_updated(wxT("spdif_devices"))) {
		_osc_control->get_value(wxT("spdif_devices"), sval);

		populate_spdif_list(sval);
	}


	if (_osc_control->is_updated(wxT("file_output"))) {
		_osc_control->get_value(wxT("file_output"), ival);
		_file_out_check->SetValue(ival != 0);
	}
	
	if (_osc_control->is_updated(wxT("channels"))) {
		_osc_control->get_value(wxT("channels"), ival);

		for (int i=0; i < _channel_choice->GetCount(); ++i) {
			if ((int)(long)_channel_choice->GetClientData(i) == ival) {
				_channel_choice->SetSelection(i);
				break;
			}
		}
		//cerr << "channels updated" << endl;
		update_conn_buttons();
	}
	if (_osc_control->is_updated(wxT("lfe"))) {
		_osc_control->get_value(wxT("lfe"), ival);
		_lfe_check->SetValue(ival != 0);
		update_conn_buttons();
	}

	if (_osc_control->is_updated(wxT("spdif_status"))) {
		_osc_control->get_value(wxT("spdif_status"), sval);
		_spdif_status_text->SetValue(sval);
	}

	if (_osc_control->is_updated(wxT("input_connections/left"))) {
		update_connection(wxT("input_connections/left"));
	}
	if (_osc_control->is_updated(wxT("input_connections/right"))) {
		update_connection(wxT("input_connections/right"));
	}
	if (_osc_control->is_updated(wxT("input_connections/center"))) {
		update_connection(wxT("input_connections/center"));
	}
	if (_osc_control->is_updated(wxT("input_connections/left_surround"))) {
		update_connection(wxT("input_connections/left_surround"));
	}
	if (_osc_control->is_updated(wxT("input_connections/right_surround"))) {
		update_connection(wxT("input_connections/right_surround"));
	}
	if (_osc_control->is_updated(wxT("input_connections/lfe"))) {
		update_connection(wxT("input_connections/lfe"));
	}

	if (_osc_control->is_updated(wxT("duration"))) {
		_osc_control->get_value(wxT("duration"), dval);
		if (dval == 0.0) {
			_duration_limit_check->SetValue(false);
			_duration_text->SetValue(wxT(""));
		}
		else {
			_duration_limit_check->SetValue(true);
			_duration_entry->SetValue(wxString::Format(wxT("%.2f"), dval));
		}

	}
	
}

void MainPanel::set_spdif_dev_choice(wxString & val)
{
	bool found = false;
	for (int i=0; i < _spdif_device_choice->GetCount(); ++i) {
		if (((StringClientData *)(wxClientData *)_spdif_device_choice->GetClientData(i))->value == val) {
			_spdif_device_choice->SetSelection(i);
			found = true;
			break;
		}
	}

	if (!found) {
		_spdif_device_choice->SetSelection(0);
	}
}

void MainPanel::populate_spdif_list(wxString & val)
{
	// each device is on its own line, "Name \t devstr"	
	int pos = 0;
	int last, tabpos;
	
	_spdif_device_choice->Clear();
	// cerr << "populate with: " << val << endl;

	_spdif_device_choice->Append(wxT("<None Selected>"), new StringClientData(wxT("")));

	while ((last = val.find('\n', pos)) >= 0) {
		wxString line = val.substr(pos, last-pos);
		if ((tabpos = line.find('\t', 0)) >= 0) {
			_spdif_device_choice->Append(line.Left(tabpos), new StringClientData(line.Mid(tabpos+1)));
		}
		pos = last+1;
	}

	wxString sval;
	_osc_control->get_value(wxT("spdif_card"), sval);
	set_spdif_dev_choice(sval);
}

void
MainPanel::OnHide(wxCommandEvent &event)
{

}


void
MainPanel::OnSize(wxSizeEvent & event)
{
	event.Skip();
}

void MainPanel::OnPaint(wxPaintEvent & event)
{
	event.Skip();
}

void
MainPanel::OnIdle(wxIdleEvent& event)
{
	if (_got_new_data) {
		// cerr << "idle update" << endl;

 		_osc_control->update_values();
		
    		update_controls();

 		_got_new_data = 0;
	}
	
	//event.Skip();
}



bool MainPanel::load_rc()
{
/*
	// open file
	string configfname(static_cast<const char *> ((_rcdir + wxFileName::GetPathSeparator() + wxT("gui_config.xml")).fn_str() ));
	XMLTree configdoc (configfname);

	if (!configdoc.initialized()) {
		fprintf (stderr, "Error loading config at %s!\n", configfname.c_str()); 
		return false;
	}

	XMLNode * rootNode = configdoc.root();
	if (!rootNode || rootNode->name() != "SLConfig") {
		fprintf (stderr, "Preset root node not found in %s!\n", configfname.c_str()); 
		return false;
	}

	XMLNode * bindingsNode = rootNode->find_named_node ("KeyBindings");
	if (!bindingsNode ) {
		fprintf(stderr, "Preset Channels node not found in %s!\n", configfname.c_str()); 
		//return false;
	}
	else {
		_keyboard->set_binding_state (*bindingsNode);
	}

	bindingsNode = rootNode->find_named_node ("SpawnConfig");
	if (!bindingsNode ) {
		fprintf(stderr, "SpawnConfig node not found in %s!\n", configfname.c_str()); 
		//return false;
	}
	else {
		_osc_control->get_spawn_config().set_state (*bindingsNode);
		_osc_control->get_default_spawn_config().set_state (*bindingsNode);
	}

*/
	return true;
}

bool MainPanel::save_rc()
{
/*
	wxString dirname = _rcdir;
	
	if ( ! wxDirExists(dirname) ) {
		if (!wxMkdir ( dirname, 0755 )) {
			printf ("Error creating %s\n", static_cast<const char *> (dirname.mb_str())); 
			return false;
		}
	}

	// make xmltree
	XMLTree configdoc;
	XMLNode * rootNode = new XMLNode("SLConfig");
	rootNode->add_property("version", sooperlooper_version);
	configdoc.set_root (rootNode);
	
	XMLNode * bindingsNode = rootNode->add_child ("KeyBindings");
	bindingsNode->add_child_nocopy (_keyboard->get_binding_state());

	bindingsNode = rootNode->add_child ("SpawnConfig");
	bindingsNode->add_child_nocopy (_osc_control->get_default_spawn_config().get_state());

	
	// write doc to file
	
	if (configdoc.write (static_cast<const char *> ((dirname + wxFileName::GetPathSeparator() + wxT("gui_config.xml")).fn_str())))
	{	    
		fprintf (stderr, "Stored settings into %s\n", static_cast<const char *> (dirname.fn_str()));
		return true;
	}
	else {
		fprintf (stderr, "Failed to store settings into %s\n", static_cast<const char *> (dirname.fn_str()));
		return false;
	}
*/
}


void MainPanel::do_load_session ()
{
	wxString filename = do_file_selector (wxT("Choose session to load"), wxT("*.ac3jack"), wxT("*.*"), wxOPEN|wxCHANGE_DIR);
	
	if ( !filename.empty() )
	{
		_osc_control->post_load_session (filename);
	}

}

void MainPanel::do_save_session ()
{
	wxString filename = do_file_selector (wxT("Choose file to save session"), wxT("slsess"), wxT("*.*"), wxSAVE|wxCHANGE_DIR|wxOVERWRITE_PROMPT);
	
	if ( !filename.empty() )
	{
		// add .slsession if there isn't one already
		if (filename.size() <= 7 || (filename.size() > 7 && filename.substr(filename.size() - 7, 7) != wxT(".slsess"))) {
			filename += wxT(".ac3jack");
		}
		_osc_control->post_save_session (filename);
	}

}

wxString MainPanel::do_file_selector(const wxString & message, const wxString & ext, const wxString & wc, int style)
{
	wxString filename;

	if (_osc_control->is_engine_local()) {
		filename = ::wxFileSelector(message, wxT(""), wxT(""), ext, wc, style);
	}
	else {
		// popup basic filename text entry
		filename = ::wxGetTextFromUser(wxString::Format(wxT("%s on remote host '%s'"), message.c_str(),
								_osc_control->get_engine_host().c_str())
					       , message);
	}

	return filename;
}


void MainPanel::on_check (wxCommandEvent &ev)
{
	int id = ev.GetId();

	if (id == ID_JackOutCheck) {
		_osc_control->post_param_change (wxT("jack_out"), (int) (_jack_out_check->GetValue() ? 1 : 0));
	}
	else if (id == ID_LfeCheck) {
		_osc_control->post_param_change (wxT("lfe"), (int) (_lfe_check->GetValue() ? 1 : 0));
		update_conn_buttons();
	}
	else if (id == ID_DurationLimitCheck) {
		//_duration_text->Enable(_duration_limit_check->GetValue());
		if (_duration_limit_check->GetValue()) {
			double duration = 0.0;
			if (_duration_entry->GetValue().ToDouble(&duration)) {
				_osc_control->post_param_change (wxT("duration"), duration);
			}
		}
		else {
			// no limit
			_osc_control->post_param_change (wxT("duration"), 0.0);
		}
	}
	else if (id == ID_JackTransportCheck) {
		_osc_control->post_param_change (wxT("jack_transport"), (int) (_jack_transport_check->GetValue() ? 1 : 0));
	}
	else if (id == ID_SpdifOutCheck) {
		_osc_control->post_param_change (wxT("spdif_out"), (int) (_spdif_out_check->GetValue() ? 1 : 0));
	}
	else if (id == ID_SpdifPcmCheck) {
		_osc_control->post_param_change (wxT("spdif_pcm"), (int) (_spdif_pcm_check->GetValue() ? 1 : 0));
	}
	else if (id == ID_FileOutCheck) {
		_osc_control->post_param_change (wxT("outfile_name"), _filename_entry->GetValue());
		_osc_control->post_param_change (wxT("file_output"), (_file_out_check->GetValue() ? 1: 0));
	}
	
	
}

void MainPanel::on_text  (wxCommandEvent &ev)
{
	int id = ev.GetId();

	if (id == ID_FilenameEntry) {
		_osc_control->post_param_change (wxT("outfile_name"), _filename_entry->GetValue());
	}
	else if (id == ID_DurationEntry) {
		double dval;
		if (_duration_entry->GetValue().ToDouble(&dval)) {
			_osc_control->post_param_change (wxT("duration"), dval);
		}
	}
}

void MainPanel::on_button (wxCommandEvent &ev)
{
	int id = ev.GetId();

	if (id == ID_BrowseButton) {
		wxString filename = do_file_selector (wxT("Choose file to save session"), wxT("slsess"), wxT("*.*"), wxSAVE|wxCHANGE_DIR|wxOVERWRITE_PROMPT);
		
		if ( !filename.empty() )
		{
			_filename_entry->SetValue(filename);
			_osc_control->post_param_change (wxT("outfile_name"), filename);
		}
	}
	else if (id == ID_LeftConnButton) {
		// popup menu
		ConnectionsPopup * popup = new ConnectionsPopup(this, _osc_control, wxT("left"));
		wxRect bounds =  _left_conn_button->GetRect();
		PopupMenu(popup, bounds.x, bounds.y + bounds.height);
	}
	else if (id == ID_RightConnButton) {
		// popup menu
		ConnectionsPopup * popup = new ConnectionsPopup(this, _osc_control, wxT("right"));
		wxRect bounds =  _right_conn_button->GetRect();
		PopupMenu(popup, bounds.x, bounds.y + bounds.height);
	}
	else if (id == ID_LeftConnButton) {
		// popup menu
		ConnectionsPopup * popup = new ConnectionsPopup(this, _osc_control, wxT("left"));
		wxRect bounds =  _left_conn_button->GetRect();
		PopupMenu(popup, bounds.x, bounds.y + bounds.height);
	}
	else if (id == ID_CenterConnButton) {
		// popup menu
		ConnectionsPopup * popup = new ConnectionsPopup(this, _osc_control, wxT("center"));
		wxRect bounds =  _center_conn_button->GetRect();
		PopupMenu(popup, bounds.x, bounds.y + bounds.height);
	}
	else if (id == ID_LeftSurrConnButton) {
		// popup menu
		ConnectionsPopup * popup = new ConnectionsPopup(this, _osc_control, wxT("left_surround"));
		wxRect bounds =  _leftsurr_conn_button->GetRect();
		PopupMenu(popup, bounds.x, bounds.y + bounds.height);
	}
	else if (id == ID_RightSurrConnButton) {
		// popup menu
		ConnectionsPopup * popup = new ConnectionsPopup(this, _osc_control, wxT("right_surround"));
		wxRect bounds =  _rightsurr_conn_button->GetRect();
		PopupMenu(popup, bounds.x, bounds.y + bounds.height);
	}
	else if (id == ID_LfeConnButton) {
		// popup menu
		ConnectionsPopup * popup = new ConnectionsPopup(this, _osc_control, wxT("lfe"));
		wxRect bounds =  _lfe_conn_button->GetRect();
		PopupMenu(popup, bounds.x, bounds.y + bounds.height);
	}

}

void MainPanel::on_choice (wxCommandEvent &ev)
{
	int id = ev.GetId();

	if (id == ID_BitrateChoice) {
		int index = _bitrate_choice->GetSelection();
		long bitrate = 0;
		if (_bitrate_choice->GetStringSelection().ToLong(&bitrate)) {
			_osc_control->post_param_change (wxT("encoder/bitrate"), (int) bitrate);
		}
	}
	else if (id == ID_SpdifDevChoice) {
		int index = _spdif_device_choice->GetSelection();
		if (index >=0) {
			_osc_control->post_param_change (wxT("spdif_card"), 
						 ((StringClientData *)(wxClientData *)_spdif_device_choice->GetClientData(index))->value);
		}
 
	}
	else if (id == ID_ChannelChoice) {
		int index = _channel_choice->GetSelection();
		int channels = (int) (long) _channel_choice->GetClientData(index);
		_osc_control->post_param_change (wxT("channels"), channels);
		update_conn_buttons();
	}

}


void MainPanel::update_conn_buttons()
{
	int channels = 0;
	int has_lfe = 0;
	
	_osc_control->get_value(wxT("channels"), channels);
	_osc_control->get_value(wxT("lfe"), has_lfe);	

	switch (channels) {
	case 5:
		_left_conn_button->Enable(true);
		_right_conn_button->Enable(true);
		_center_conn_button->Enable(true);
		_leftsurr_conn_button->Enable(true);			
		_rightsurr_conn_button->Enable(true);
		break;
	case 4:
		_left_conn_button->Enable(true);
		_right_conn_button->Enable(true);
		_center_conn_button->Enable(false);
		_leftsurr_conn_button->Enable(true);			
		_rightsurr_conn_button->Enable(true);
		break;
	case 3:
		_left_conn_button->Enable(true);
		_right_conn_button->Enable(true);
		_center_conn_button->Enable(true);
		_leftsurr_conn_button->Enable(false);			
		_rightsurr_conn_button->Enable(false);
		break;
	case 2:
		_left_conn_button->Enable(true);
		_right_conn_button->Enable(true);
		_center_conn_button->Enable(false);
		_leftsurr_conn_button->Enable(false);			
		_rightsurr_conn_button->Enable(false);
		break;
	default:
		_left_conn_button->Enable(false);
		_right_conn_button->Enable(false);
		_center_conn_button->Enable(false);
		_leftsurr_conn_button->Enable(false);			
		_rightsurr_conn_button->Enable(false);
		break;
	}
		
	_lfe_conn_button->Enable (has_lfe != 0 ? true: false);
}

void MainPanel::update_connection(const wxString & param)
{
	wxString value;
	wxButton * button;
	
	_osc_control->get_value(param, value);

	if (value.empty()) {
		value = wxT("<Not Connected>");
	}
	
	if (param == wxT("input_connections/left")) {
		button = _left_conn_button;
	}
	else if (param == wxT("input_connections/center")) {
		button = _center_conn_button;
	}
	else if (param == wxT("input_connections/right")) {
		button = _right_conn_button;
	}
	else if (param == wxT("input_connections/left_surround")) {
		button = _leftsurr_conn_button;
	}
	else if (param == wxT("input_connections/right_surround")) {
		button = _rightsurr_conn_button;
	}
	else if (param == wxT("input_connections/lfe")) {
		button = _lfe_conn_button;
	}

	// convert newlines to ", "
	value.Trim();
	value.Replace(wxT("\n"), wxT(", "));
	button->SetLabel(value);
	button->SetToolTip(value);
	
}
