/*
** Copyright (C) 2004 Jesse Chappell <jesse@essej.net>
**  
** This program is free software; you can redistribute it and/or modify
** it under the terms of the GNU General Public License as published by
** the Free Software Foundation; either version 2 of the License, or
** (at your option) any later version.
**  
** This program is distributed in the hope that it will be useful,
** but WITHOUT ANY WARRANTY; without even the implied warranty of
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
** GNU General Public License for more details.
**  
** You should have received a copy of the GNU General Public License
** along with this program; if not, write to the Free Software
** Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
**  
*/

#ifndef __ac3jack_mainpanel__
#define __ac3jack_mainpanel__


#include <wx/wx.h>

#include <string>
#include <vector>
#include <iostream>

#include <sigc++/object.h>
#include <sigc++/signal.h>
#include <sigc++/connection.h>

class wxSpinCtrl;

namespace AC3JACKGui {

class OscClient;
	
class MainPanel
	: public wxPanel,  public SigC::Object
{
public:
	
	// ctor(s)
	MainPanel(wxWindow * parent, wxWindowID id, const wxPoint& pos, const wxSize& size);
	virtual ~MainPanel();
	
	// event handlers (these functions should _not_ be virtual)
	//void OnQuit(wxCommandEvent& event);
	void OnHide(wxCommandEvent &event);
	//void OnClose(wxCloseEvent &event);
	
	void OnSize(wxSizeEvent & event);
	void OnPaint(wxPaintEvent & event);
	
	void OnIdle(wxIdleEvent& event);
	void OnUpdateTimer(wxTimerEvent &ev);
	void OnActivate(wxActivateEvent &ev);
	
	OscClient & get_osc_control() { return *_osc_control; }
	
	void do_close(bool quitengine=false);
	void do_load_session ();
	void do_save_session ();
	
	
	bool load_rc();
	bool save_rc();

	wxString do_file_selector(const wxString & message, const wxString & ext, const wxString & wc, int style);
	
	SigC::Signal2<void,int, int> PreferredSizeChange;

	
protected:

	void init();

	void osc_data_ready();
	
	void on_view_menu (wxCommandEvent &ev);

	void on_about (wxCommandEvent &ev);
	void on_help (wxCommandEvent &ev);

	void on_check (wxCommandEvent &ev);
	void on_text  (wxCommandEvent &ev);
	void on_button (wxCommandEvent &ev);
	void on_choice (wxCommandEvent &ev);
	
	void update_controls();

	void on_connect_failed (const std::string & msg);
	void on_connection_lost (const std::string & msg);
	void on_engine_alive ();
	void on_error_received (const std::string & msg);

	void time_to_string(double time, wxString & retval);
	void populate_spdif_list(wxString & val);
	void set_spdif_dev_choice(wxString & val);
	void update_conn_buttons();
	void update_connection(const wxString & param);

	OscClient * _osc_control;
	SigC::Connection  _osc_update_connection;
	
	wxScrolledWindow * _scroller;
	wxBoxSizer * _main_sizer;
	wxBoxSizer * _topsizer;
	wxPanel    * _top_panel;

	// controls

	wxChoice * _channel_choice;
	wxComboBox * _bitrate_choice;
	wxCheckBox * _lfe_check;
	wxButton   * _left_conn_button;
	wxButton   * _right_conn_button;
	wxButton   * _center_conn_button;
	wxButton   * _leftsurr_conn_button;
	wxButton   * _rightsurr_conn_button;
	wxButton   * _lfe_conn_button;	
	
	wxCheckBox * _file_out_check;
	wxTextCtrl * _filename_entry;
	wxButton   * _browse_button;
	wxCheckBox * _duration_limit_check;
	wxTextCtrl * _duration_entry;
	wxTextCtrl * _filesize_text;
	wxTextCtrl * _duration_text;	
	wxCheckBox * _jack_transport_check;
	
	wxCheckBox    * _jack_out_check;
	wxCheckBox    * _spdif_out_check;
	wxCheckBox    * _spdif_pcm_check;
	wxTextCtrl    * _spdif_status_text;
	wxChoice      * _spdif_device_choice;
	
	wxTimer * _update_timer;

	int     _got_new_data;
	wxString  _rcdir;
	bool _engine_alive;
	
private:
    // any class wishing to process wxWindows events must use this macro
    DECLARE_EVENT_TABLE()
	
};

class StringClientData
	: public wxClientData
{
public:
	StringClientData(const wxString & val) : value(val) {}
	virtual ~StringClientData() { /* std::cerr << "string " << value.c_str() << " destructing" << std::endl; */}
	wxString value;
};
	
}

#endif
