/*
** Copyright (C) 2004 Jesse Chappell <jesse@essej.net>
**  
** This program is free software; you can redistribute it and/or modify
** it under the terms of the GNU General Public License as published by
** the Free Software Foundation; either version 2 of the License, or
** (at your option) any later version.
**  
** This program is distributed in the hope that it will be useful,
** but WITHOUT ANY WARRANTY; without even the implied warranty of
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
** GNU General Public License for more details.
**  
** You should have received a copy of the GNU General Public License
** along with this program; if not, write to the Free Software
** Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
**  
*/

#ifndef __ac3jack_gui_loop_control__
#define __ac3jack_gui_loop_control__

#include <wx/string.h>
#include <wx/timer.h>
#include <sigc++/sigc++.h>
#include <map>
#include <vector>
#include <string>

#include <lo/lo.h>

// #include <pbd/xml++.h>
// #include "plugin.hpp"


namespace AC3JACKGui {

class OscUpdateTimer;	


class OscClient
	: public SigC::Object
{
  public:

	// ctor(s)
	OscClient (const wxString & rcdir=wxT(""));
	virtual ~OscClient();

	void set_host (wxString host) { _host = host; }
	void set_port (int port) { _port = port; }
	
	bool connect();
	bool disconnect(bool killit=false);
	
	bool post_param_change (wxString ctrl, float val);
	bool post_param_change (wxString ctrl, double val);
	bool post_param_change (wxString ctrl, int64_t val);
	bool post_param_change (wxString ctrl, const wxString & val);
	bool post_param_change (wxString ctrl, int val);
	
	bool post_save_session(wxString fname);
	bool post_load_session(wxString fname);
	
	bool is_engine_local();
	wxString get_engine_host() const { return _host; }
	int get_engine_port() const { return _port; }
	bool connected() const { return (_osc_addr != 0 && _pingack); }
	bool we_spawned() const { return _we_spawned; }
	
	void request_values ();
	void request_param_value (wxString ctrl);

	void update_values();

	void send_quit();
	void send_alive_ping();
	
	bool is_updated (wxString ctrl);
	
	bool get_value (wxString ctrl, float &retval);
	bool get_value (wxString ctrl, double &retval);
	bool get_value (wxString ctrl, int &retval);
	bool get_value (wxString ctrl, int64_t &retval);
	bool get_value (wxString ctrl, wxString &retval);

	void pingtimer_expired();

	SigC::Signal1<void,int> ServerConnected;
	SigC::Signal1<void, const std::string&> ConnectFailed;
	SigC::Signal1<void, const std::string&> LostConnection;
	SigC::Signal1<void, const std::string&> ErrorReceived;
	SigC::Signal0<void> Disconnected;
	SigC::Signal0<void> NewDataReady;
	SigC::Signal0<void> IsAlive;
	
  protected:
	
	static int _param_handler(const char *path, const char *types, lo_arg **argv, int argc,
				    void *data, void *user_data);

	int param_handler(const char *path, const char *types, lo_arg **argv, int argc, void *data);

	static int _pingack_handler(const char *path, const char *types, lo_arg **argv, int argc,
				    void *data, void *user_data);

	int pingack_handler(const char *path, const char *types, lo_arg **argv, int argc, void *data);

	static int _alive_handler(const char *path, const char *types, lo_arg **argv, int argc,
				    void *data, void *user_data);

	int alive_handler(const char *path, const char *types, lo_arg **argv, int argc, void *data);

	static int _error_handler(const char *path, const char *types, lo_arg **argv, int argc,
				    void *data, void *user_data);

	int error_handler(const char *path, const char *types, lo_arg **argv, int argc, void *data);
	
	static void * _osc_traffic(void * arg);
	void osc_traffic();
	
	void setup_param_map();

	bool spawn_looper();
	
	std::string   _osc_url;
	lo_address _osc_addr;

	lo_server  _osc_server;
	std::string  _our_url;

/*
	typedef std::map<wxString, float> ControlValMap;
	typedef std::vector<ControlValMap> ControlValMapList;
        ControlValMapList _params_val_map;
	ControlValMap     _global_val_map;
	
	std::map<SooperLooper::LooperState, wxString> state_map;

	typedef std::vector<ControlValMap> UpdatedCtrlMapList;
	UpdatedCtrlMapList _updated;
*/

	typedef std::map<wxString, bool> UpdatedCtrlMap;
	UpdatedCtrlMap     _params_updated;

	

	typedef std::map<wxString,int> IntControlMap;
	typedef std::map<wxString,int64_t> Int64ControlMap;
	typedef std::map<wxString,float> FloatControlMap;
	typedef std::map<wxString,double> DoubleControlMap;
	typedef std::map<wxString,wxString> StringControlMap;	
	
	
	IntControlMap _params_int_map;
	Int64ControlMap _params_int64_map;
        FloatControlMap _params_float_map;
	DoubleControlMap _params_double_map;
	StringControlMap _params_string_map;	
	
	wxString _host;
	int      _port;
	bool     _force_spawn;
	wxString _rcdir;
	
	OscUpdateTimer * _updatetimer;
	bool _pingack;
	int  _waiting;
	bool _failed;
	bool _lastchance;
	bool _we_spawned;
	
	long _engine_pid;

	bool init_traffic_thread();
	void terminate_traffic_thread();
	pthread_t _osc_traffic_thread;
	int       _traffic_request_pipe[2];
	volatile bool      _traffic_done;
	volatile bool      _sentinel;
		
};

class OscUpdateTimer : public wxTimer
{
public:
	OscUpdateTimer(OscClient *oscctrl): wxTimer(), _oscctrl(oscctrl) {}
	
	OscClient * _oscctrl;
		
	void Notify() {
		_oscctrl->pingtimer_expired();
	}
};


	

	
};

#endif
