/*
** Copyright (C) 2006 Jesse Chappell <jesse@essej.net>
**  
** This program is free software; you can redistribute it and/or modify
** it under the terms of the GNU General Public License as published by
** the Free Software Foundation; either version 2 of the License, or
** (at your option) any later version.
**  
** This program is distributed in the hope that it will be useful,
** but WITHOUT ANY WARRANTY; without even the implied warranty of
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
** GNU General Public License for more details.
**  
** You should have received a copy of the GNU General Public License
** along with this program; if not, write to the Free Software
** Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
**  
*/


#ifndef __ac3jack_threadwriter__
#define __ac3jack_threadwriter__

#include <string>
#include "boost/thread/thread.hpp"
#include "boost/thread/condition.hpp"

namespace AC3JACK
{

class ThreadWriter
{
  public:

	ThreadWriter();
	virtual ~ThreadWriter();

	virtual void start();

	virtual bool ok() const { return _ok; }

	/**
	 *   queues data to be written
	 *   this method is realtime safe
	 */
	virtual size_t write(unsigned char * data, size_t length) = 0;
	
  protected:

	virtual void run_work_thread() = 0;
	
	boost::thread   * _butler_thread;
	boost::condition  _more_incoming;
	boost::try_mutex  _butler_mutex;

	volatile bool     _active;
	bool              _ok;


	class Butler
        {
           public:
                Butler(ThreadWriter * eng) : parent(eng) {}
                void operator()() { parent->run_work_thread(); }
                
           protected:
                ThreadWriter * parent;
        };

};



	

}


#endif
